library sadad_payment_flutter;

import 'dart:ui';
import 'dart:io';

import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:get/get.dart';
import 'package:network_info_plus/network_info_plus.dart';
import 'package:pay/pay.dart';
import 'package:sadad_payment_flutter/apputils/app_formatter.dart';
import 'package:sadad_payment_flutter/apputils/appassets.dart';
import 'package:sadad_payment_flutter/apputils/appcolors.dart';
import 'package:sadad_payment_flutter/apputils/appdialogs.dart';
import 'package:sadad_payment_flutter/assets/locale/arabic.dart';
import 'package:sadad_payment_flutter/assets/locale/english.dart';
import 'package:sadad_payment_flutter/commonWidgets.dart';
import 'package:sadad_payment_flutter/controller.dart';
import 'package:sadad_payment_flutter/model/checkedallowedcountrymodel.dart';
import 'package:sadad_payment_flutter/model/creditcardsettingsmodel.dart';
import 'package:sadad_payment_flutter/model/transactionIdDetailsModel.dart';
import 'package:sadad_payment_flutter/model/usermetapreference.dart';
import 'package:sadad_payment_flutter/model/webViewDetailsModel.dart';
import 'package:sadad_payment_flutter/payment_web_view.com.dart';

import 'package:sadad_payment_flutter/services/appservices.dart';
import 'package:sms_autofill/sms_autofill.dart';

Locale selectedLanguage = Get.deviceLocale ?? const Locale("en");

class PaymentScreen extends StatefulWidget {
  Color? themeColor;
  Color? paymentButtonColor;
  Color? paymentButtonTextColor;
  List<PaymentType>? paymentTypes;
  String token;
  String mobile;
  String email;
  String customerName;
  double amount;
  Image image;
  String? titleText;
  List<Map> productDetail;
  PackageMode? packageMode;

  PaymentScreen(
      {super.key,
      this.themeColor = const Color(0xFFA02058),
      required this.customerName,
      this.titleText,
      this.paymentButtonColor,
      this.paymentButtonTextColor,
      required this.mobile,
      required this.token,
      required this.email,
      required this.productDetail,
      required this.amount,
      this.packageMode = PackageMode.debug,
      required this.image,
      this.paymentTypes = const [PaymentType.creditCard, PaymentType.debitCard, PaymentType.sadadPay]});

  @override
  State<PaymentScreen> createState() => _PaymentScreenState();
}

class _PaymentScreenState extends State<PaymentScreen> {
  Color primaryColor = AppColors.primaryColor;
  MainPageController controller = Get.put(MainPageController());
  List<CreditCardSettingsModel> creditCardSettingsModel = [];
  UserMetaPreference? userMetaPreference;
  String credit_card_bankpage_type = "";
  String is_american_express = "";
  String is_cybersourse_visa = "";
  String is_cybersourse_mastercard = "";

  String checkSum = "";

  @override
  void initState() {
    super.initState();
    setLanguage();
    widget.amount = double.parse(widget.amount.toStringAsFixed(2));

    creditCardSettings(token: widget.token);
    controller.sadadPayPasswordController.clear();
    controller.cardHolderNameController.clear();
    controller.sadadPayOTPController.clear();
    controller.expiryController.clear();
    controller.cvvController.clear();
    controller.sadadPayCellNumberController.clear();
    controller.cardNumberController.clear();
    primaryColor = widget.themeColor!;
    if (widget.paymentTypes!.isEmpty) {
      widget.paymentTypes = [PaymentType.creditCard, PaymentType.debitCard, PaymentType.sadadPay];
    } else {
      controller.selectedPaymentMethod = widget.paymentTypes!.first;
    }
  }

  @override
  Widget build(BuildContext context) {
    if (widget.titleText!.length > 30) {
      throw "Title text must be short";
    }
    return GetBuilder(
        init: MainPageController(),
        builder: (MainPageController controller) {
          return GestureDetector(
            onTap: () {
              FocusScope.of(context).unfocus();
            },
            child: Scaffold(
                backgroundColor: const Color(0xFFFAFAFA),
                body: SafeArea(
                  top: false,
                  child: SingleChildScrollView(
                    physics: const ClampingScrollPhysics(),
                    child: Column(
                      children: [topContainer(), middleContainer(), bottomContainer()],
                    ),
                  ),
                )),
          );
        });
  }

  Widget middleContainer() {
    MainPageController controller = Get.find<MainPageController>();
    return Padding(
      padding: const EdgeInsets.all(12),
      child: Column(
        children: [
          const SizedBox(height: 8),
          if (widget.paymentTypes!.contains(PaymentType.wallet))
            Row(
              children: [
                const Expanded(
                  child: Divider(
                    endIndent: 15,
                    indent: 50,
                    thickness: 2,
                    height: 10,
                    color: Color(0xFFE4E4E4),
                  ),
                ),
                Text("Or Pay Using".tr, style: const TextStyle(color: AppColors.black, fontSize: 12)),
                const Expanded(
                  child: Divider(
                    endIndent: 50,
                    indent: 15,
                    thickness: 2,
                    height: 10,
                    color: AppColors.dividerColor,
                  ),
                )
              ],
            ),
          const SizedBox(height: 20),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              if (widget.paymentTypes!.contains(PaymentType.creditCard)) cardContainer(text: "Credit Card".tr, type: PaymentType.creditCard, imagePath: AssetPath.creditCard),
              if (widget.paymentTypes!.contains(PaymentType.debitCard)) cardContainer(text: "Debit Card".tr, type: PaymentType.debitCard, imagePath: AssetPath.debitIcon),
              if (widget.paymentTypes!.contains(PaymentType.sadadPay)) cardContainer(text: "Sadad Pay".tr, type: PaymentType.sadadPay, imagePath: AssetPath.sadadWallet)
            ],
          ),
          const SizedBox(height: 12),
          Directionality(
              textDirection: selectedLanguage == const Locale("ar") ? TextDirection.rtl : TextDirection.ltr, child: getSelectedPaymentMethod(selectedMethod: controller.selectedPaymentMethod))
        ],
      ),
    );
  }

  Widget topContainer() {
    return SizedBox(
      height: Get.width * 0.56 + kToolbarHeight,
      width: double.infinity,
      child: Stack(children: [
        Container(
          padding: const EdgeInsets.only(top: kToolbarHeight),
          height: Get.width * 0.5 + kToolbarHeight,
          decoration: BoxDecoration(color: primaryColor, borderRadius: const BorderRadius.only(bottomLeft: Radius.circular(60), bottomRight: Radius.circular(60))),
          child: Padding(
            padding: const EdgeInsets.all(12),
            child: Column(mainAxisAlignment: MainAxisAlignment.spaceBetween, children: [
              Row(
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Expanded(
                    child: Align(
                      alignment: Alignment.centerLeft,
                      child: GestureDetector(
                        onTap: () {
                          Navigator.pop(context);
                        },
                        child: const Icon(
                          Icons.arrow_back_ios_new_outlined,
                          color: Colors.white,
                        ),
                      ),
                    ),
                  ),
                  Column(
                    children: [
                      Container(
                        constraints: BoxConstraints(minWidth: MediaQuery.of(context).size.width * 0.16, maxWidth: MediaQuery.of(context).size.width * 0.43),
                        height: MediaQuery.of(context).size.width * 0.16,
                        padding: const EdgeInsets.all(8),
                        decoration: BoxDecoration(color: AppColors.white, borderRadius: BorderRadius.circular(6)),
                        child: widget.image,
                      ),
                      const SizedBox(height: 9),
                      Text(
                        widget.titleText ?? "",
                        style: const TextStyle(color: AppColors.white),
                      )
                    ],
                  ),
                  Expanded(
                    child: PopupMenuButton(
                      padding: EdgeInsets.zero,
                      constraints: const BoxConstraints(maxWidth: 100),
                      position: PopupMenuPosition.under,
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.end,
                        children: [
                          Text(selectedLanguage.languageCode == "en" ? "En" : "Ar", style: const TextStyle(color: Colors.white, fontSize: 12)),
                          const Icon(Icons.keyboard_arrow_down_rounded, color: AppColors.dividerColor)
                        ],
                      ),
                      itemBuilder: (context) {
                        return [
                          commonPopupMenuItem(
                              text: "English",
                              onTap: () {
                                selectedLanguage = const Locale("en");
                                Get.updateLocale(selectedLanguage);
                                controller.update();
                              },
                              value: "en"),
                          commonPopupMenuItem(
                              text: "Arabic",
                              onTap: () {
                                selectedLanguage = const Locale("ar");
                                Get.updateLocale(selectedLanguage);
                                controller.update();
                              },
                              value: "ar"),
                        ];
                      },
                    ),
                  ),
                  // Expanded(
                  //   child: InkWell(
                  //     onTap: () {},
                  //     child: Row(
                  //       mainAxisAlignment: MainAxisAlignment.end,
                  //       children: const [Text("Eng", style: TextStyle(color: Colors.white, fontSize: 12)), Icon(Icons.keyboard_arrow_down_rounded, color: AppColors.white)],
                  //     ),
                  //   ),
                  // )
                ],
              ),
              const SizedBox(height: 0),
              Container(
                  padding: const EdgeInsets.symmetric(horizontal: 16),
                  decoration: BoxDecoration(color: AppColors.black54, borderRadius: BorderRadius.circular(20)),
                  height: 40,
                  child: Row(mainAxisSize: MainAxisSize.min, children: [
                    Text(
                      "Select an option to Pay ".tr,
                      style: const TextStyle(color: AppColors.white, fontSize: 12),
                    ),
                    priceWidget(price: widget.amount.toStringAsFixed(2).toString())
                  ])),
              const SizedBox(height: 25)
            ]),
          ),
        ),
        if (widget.paymentTypes!.contains(PaymentType.wallet))
          widget.packageMode == PackageMode.debug
              ? Container(color: widget.themeColor, height: 20, width: 20)
              : Align(
                  alignment: Alignment.bottomCenter,
                  child: Platform.isIOS
                      ? InkWell(
                          onTap: () {
                            // if let merchantID = arrUserMetaPreferences["userId"] as? Int{
                            // UserDefaults.standard.set(merchantID, forKey: "MerchantUserID")
                            // }
                            // if let arrUsers = arrUserMetaPreferences["user"] as? [String:Any], let merchantID = arrUsers["SadadId"] as? String{
                            // UserDefaults.standard.set(merchantID, forKey: "MerchantSadadID")
                            // }
                            if (userMetaPreference?.user?.sadadId == "" || userMetaPreference?.userId == "") {
                              showDialog(
                                  context: context,
                                  builder: (context) {
                                    return Dialog(
                                      child: Column(
                                        children: [
                                          Text("Merchant has no permission to use SDK. Please contact to administrator.".tr),
                                          ElevatedButton(
                                              onPressed: () {
                                                Navigator.pop(context);
                                              },
                                              child: Text("Okay".tr))
                                        ],
                                      ),
                                    );
                                  });
                            } else {
                              WebViewDetailsModel webViewDetailsModel = WebViewDetailsModel(
                                merchantSadadId: userMetaPreference?.user?.sadadId,
                                checksum: checkSum,
                                merchantUserId: userMetaPreference?.userId,
                                cardHolderName: "cardholdername",
                                email: widget.email,
                                paymentMethod: "applePay",
                                contactNumber: widget.mobile,
                                transactionAmount: widget.amount,
                                productDetail: widget.productDetail,
                                transactionId: "",
                                token: widget.token,
                              );
                              Navigator.push(context, MaterialPageRoute(
                                builder: (context) {
                                  return PaymentWebViewScreen(webViewDetailsModel: webViewDetailsModel);
                                },
                              )).then((value) {
                                Navigator.pop(context, value);
                              });
                            }
                          },
                          child: Container(
                            decoration: BoxDecoration(color: AppColors.black, borderRadius: BorderRadius.circular(5)),
                            height: 55,
                            margin: const EdgeInsets.symmetric(horizontal: 20),
                            child: Center(child: Image.asset(AssetPath.applePay, package: 'sadad_payment_flutter', height: 22)),
                          ),
                        )
                      : /*InkWell(
                          onTap: () {
                            if (userMetaPreference?.user?.sadadId == "" || userMetaPreference?.userId == "") {
                              showDialog(
                                  context: context,
                                  builder: (context) {
                                    return Dialog(
                                      child: Column(
                                        children: [
                                          Text("Merchant has no permission to use SDK. Please contact to administrator.".tr),
                                          ElevatedButton(
                                              onPressed: () {
                                                Navigator.pop(context);
                                              },
                                              child: Text("Okay".tr))
                                        ],
                                      ),
                                    );
                                  });
                            } else {
                              WebViewDetailsModel webViewDetailsModel = WebViewDetailsModel(
                                merchantSadadId: userMetaPreference?.user?.sadadId,
                                checksum: checkSum,
                                merchantUserId: userMetaPreference?.userId,
                                cardHolderName: "cardholdername",
                                email: widget.email,
                                paymentMethod: "googlepay",
                                contactNumber: widget.mobile,
                                transactionAmount: widget.amount,
                                productDetail: widget.productDetail,
                                transactionId: "",
                                token: widget.token,
                              );
                              Navigator.push(context, MaterialPageRoute(
                                builder: (context) {
                                  return PaymentWebViewScreen(webViewDetailsModel: webViewDetailsModel);
                                },
                              )).then((value) {
                                Navigator.pop(context, value);
                              });
                            }
                          },
                          child: Container(
                            decoration: BoxDecoration(
                                color: AppColors.white, borderRadius: BorderRadius.circular(5), boxShadow: const [BoxShadow(color: AppColors.black12, blurRadius: 5, blurStyle: BlurStyle.outer)]),
                            height: 55,
                            margin: const EdgeInsets.symmetric(horizontal: 20),
                            child: Center(child: Image.asset(AssetPath.gPay, package: 'sadad_payment_flutter', height: 22)),
                          ),
                        )*/
                      GooglePayButton(
                          paymentConfiguration: PaymentConfiguration.fromJsonString('''{
   "provider":"google_pay",
   "data":{
      "environment":"TEST",
      "apiVersion":2,
      "apiVersionMinor":0,
      "allowedPaymentMethods":[
         {
            "type":"CARD",
            "tokenizationSpecification":{
               "type":"PAYMENT_GATEWAY",
               "parameters":{
                  "gateway":"mpgs",
                  "gatewayMerchantId":"SPSQNB01"
               }
            },
            "parameters":{
               "allowedCardNetworks":[
                  "VISA",
                  "MASTERCARD"
               ],
               "allowedAuthMethods":[
                  "PAN_ONLY",
                  "CRYPTOGRAM_3DS"
               ],
               "billingAddressRequired":false,
               "billingAddressParameters":{
                  "format":"FULL",
                  "phoneNumberRequired":true
               }
            }
         }
      ],
      "transactionInfo":{
         "countryCode":"QA",
         "currencyCode":"QAR"
      }
   }
}'''),
                          paymentItems: [
                            const PaymentItem(
                              label: 'Total',
                              amount: '99.99',
                              status: PaymentItemStatus.final_price,
                              type: PaymentItemType.total,
                            )
                          ],
                          type: GooglePayButtonType.pay,
                          margin: const EdgeInsets.only(top: 15.0),
                          onPaymentResult: (result) {
                            print("Payment result ${result}");
                          },
                          loadingIndicator: const Center(
                            child: CircularProgressIndicator(),
                          ),
                        ),
                )
      ]),
    );
  }

  PopupMenuItem commonPopupMenuItem({required Function() onTap, required String value, required String text}) {
    return PopupMenuItem(
      padding: const EdgeInsets.only(left: 10),
      height: 30,
      onTap: onTap,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        mainAxisSize: MainAxisSize.min,
        children: [
          SizedBox(
            height: 20,
            width: 20,
            child: Radio(
              activeColor: widget.themeColor,
              value: value,
              groupValue: selectedLanguage.languageCode,
              onChanged: null,
              fillColor: MaterialStateProperty.all(widget.themeColor),
            ),
          ),
          const SizedBox(width: 5),
          Text(
            text,
            style: const TextStyle(fontSize: 14),
          ),
        ],
      ),
    );
  }

  Widget priceWidget({required String price}) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Text(price, style: const TextStyle(color: AppColors.white, fontSize: 15, fontWeight: FontWeight.w900)),
        const Text("QAR", style: TextStyle(color: AppColors.white, fontSize: 10, fontWeight: FontWeight.w900, fontFeatures: [FontFeature.superscripts()])),
      ],
    );
  }

  Widget cardContainer({required String text, required String imagePath, required PaymentType type}) {
    MainPageController controller = Get.find<MainPageController>();
    return Expanded(
      child: GestureDetector(
        onTap: () {
          controller.selectedPaymentMethod = type;
          controller.update();
        },
        child: Container(
          margin: const EdgeInsets.symmetric(horizontal: 10),
          clipBehavior: Clip.hardEdge,
          foregroundDecoration:
              BoxDecoration(borderRadius: BorderRadius.circular(5), boxShadow: const [BoxShadow(color: AppColors.boxShadow, offset: Offset(0, 3), blurStyle: BlurStyle.inner, blurRadius: 5)]),
          height: 60,
          decoration: BoxDecoration(borderRadius: BorderRadius.circular(5), color: AppColors.white, boxShadow: const [BoxShadow(color: Colors.black12, blurRadius: 5, blurStyle: BlurStyle.outer)]),
          child: Stack(
            alignment: Alignment.bottomCenter,
            children: [
              Padding(
                padding: const EdgeInsets.only(left: 16, right: 16, top: 6),
                child: Column(mainAxisAlignment: MainAxisAlignment.spaceBetween, crossAxisAlignment: CrossAxisAlignment.center, children: [
                  widget.packageMode == PackageMode.debug
                      ? Container(color: widget.themeColor, height: 20, width: 20)
                      : ImageIcon(AssetImage(imagePath, package: 'sadad_payment_flutter'), size: 25, color: controller.selectedPaymentMethod == type ? primaryColor : AppColors.grey),
                  FittedBox(fit: BoxFit.scaleDown, child: Text(text, style: const TextStyle(fontSize: 12, fontWeight: FontWeight.w500))),
                  Container(height: 8)
                ]),
              ),
              Container(height: 7.5, color: controller.selectedPaymentMethod == type ? primaryColor : AppColors.transparent)
            ],
          ),
        ),
      ),
    );
  }

  getSelectedPaymentMethod({required PaymentType selectedMethod}) {
    switch (selectedMethod) {
      case PaymentType.creditCard:
        return creditCardContainer();
      case PaymentType.debitCard:
        return debitCardContainer();
      case PaymentType.sadadPay:
        return sadadPayContainer();
      case PaymentType.wallet:
        // TODO: Handle this case.
        break;
    }
  }

  Widget creditCardContainer() {
    return Container(
      margin: const EdgeInsets.all(10),
      padding: const EdgeInsets.all(20),
      decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(5),
          color: AppColors.white,
          border: Border.all(color: AppColors.borderColor),
          boxShadow: const [BoxShadow(color: AppColors.black12, offset: Offset(0, 0), blurRadius: 5, blurStyle: BlurStyle.outer)]),
      child: Column(mainAxisSize: MainAxisSize.min, children: [
        // Row(
        //   children: [
        //     widget.packageMode == PackageMode.debug
        //         ? Container(height: 20, width: 20, color: widget.themeColor)
        //         : ImageIcon(const AssetImage(AssetPath.creditCard, package: 'sadad_payment_flutter'), color: widget.themeColor),
        //     const SizedBox(width: 16),
        //     Text(
        //       "Credit Card".tr,
        //       style: TextStyle(color: primaryColor, fontSize: 15, fontWeight: FontWeight.bold),
        //     ),
        //   ],
        // ),
        // const SizedBox(height: 15),

        Focus(
          onFocusChange: (hasFocus) {
            if (controller.cardHolderNameController.text != "") {
              controller.creditCardHolderForm.currentState?.validate();
            }
          },
          child: Form(
            key: controller.creditCardHolderForm,
            child: CommonWidgets.commonUnderLineTextField(
                helperText: null,
                validator: (p0) {
                  if (p0!.isEmpty) {
                    return "Please enter card holder name".tr;
                  } else if (p0.split(" ").length < 2) {
                    return "Card holder name is invalid.".tr;
                  } else {
                    return null;
                  }
                },
                controller: controller.cardHolderNameController,
                labelText: "Card Holder Name".tr,
                keyboardType: TextInputType.text,
                inputFormatters: [FilteringTextInputFormatter.allow(RegExp('[a-zA-Z ]'))],
                themeColor: widget.themeColor ?? primaryColor,
                maxlength: 100),
          ),
        ),
        Obx(() => Focus(
              onFocusChange: (hasFocus) {
                if (controller.cardNumberController.text != "") {
                  controller.creditCardNumberForm.currentState?.validate();
                }
              },
              child: Form(
                key: controller.creditCardNumberForm,
                child: CommonWidgets.commonUnderLineTextField(
                    helperText: null,
                    keyboardType: const TextInputType.numberWithOptions(decimal: false, signed: false),
                    validator: (p0) {
                      controller.cardType.value = controller.getCardType(p0!);
                      if (p0.isEmpty) {
                        return "Please enter card number".tr;
                      } else if (controller.cardType.value == "DINERS_CLUB" ||
                          controller.cardType.value == "MAESTRO" ||
                          controller.cardType.value == "JCB" ||
                          controller.cardType.value == "Union Pay Card" ||
                          controller.cardType.value == "Discover Card") {
                        return "This card is not supported".tr;
                      } else if ((p0.length < 19 && controller.cardType.value != "Amex" || p0.length < 18 && controller.cardType.value == "Amex") ||
                          (controller.cardType.value == "Visa" && p0.length < 16) ||
                          (controller.cardType.value == "Mastercard" && p0.length < 16)) {
                        return "Please enter a valid credit card number.".tr;
                      } else if (controller.cardType.value == "") {
                        return "This is not a valid credit card number".tr;
                      } else {
                        return null;
                      }
                    },
                    inputFormatters: [LengthLimitingTextInputFormatter(19), CreditCardFormatter()],
                    controller: controller.cardNumberController,
                    maxlength: 19,
                    labelText: "Card Number".tr,
                    onChanged: (cardNumber) {
                      controller.cardType.value = controller.getCardType(cardNumber);
                      print(controller.cardType.value);
                    },
                    //onChanged:
                    suffixIcon: controller.getImagePath(controller.cardType.value) == ""
                        ? const SizedBox()
                        : Image.asset(controller.getImagePath(controller.cardType.value), package: 'sadad_payment_flutter', height: 19),
                    themeColor: widget.themeColor ?? primaryColor),
              ),
            )),
        Row(
          children: [
            Expanded(
              child: Focus(
                onFocusChange: (hasFocus) {
                  if (controller.expiryController.text != "") {
                    controller.expiryValidityForm.currentState?.validate();
                  }
                },
                child: Form(
                  key: controller.expiryValidityForm,
                  child: CommonWidgets.commonUnderLineTextField(
                      hintText: "MM/YY",
                      keyboardType: const TextInputType.numberWithOptions(decimal: false, signed: false),
                      inputFormatters: [
                        FilteringTextInputFormatter.allow(RegExp('[0-9]')),
                        ExpiryDateFormatter() /*TextInputFormatter.withFunction((oldValue, newValue) {
                          if (newValue.text.length > 2 && (newValue.text.contains("/") == false)) {
                            print("base:::${newValue.selection.baseOffset}");
                            print("${newValue.selection.baseOffset}");
                            String text = newValue.text.replaceAllMapped(RegExp(r".{2}"), (match) => "${match.group(0)}/");
                            return TextEditingValue(
                                text: text,
                                selection: TextSelection.collapsed(offset: (oldValue.text.length < newValue.text.length) ? newValue.selection.baseOffset + 1 : newValue.selection.baseOffset - 1),
                                composing: newValue.composing);
                          } else {
                            return newValue;
                          }
                        })*/
                      ],
                      validator: (p0) {
                        if (p0!.isEmpty) {
                          return "Please enter expiry.".tr;
                        } else if (p0.length < 5) {
                          return "Enter valid expiry.".tr;
                        } else if (int.parse(p0.split("/").first) > 12 || int.parse(p0.split("/").first) < 1) {
                          return "Please enter valid expiry.".tr;
                        } else {
                          int currentYear = DateTime.now().year;
                          int enteredYear = int.parse(currentYear.toString()[0] + currentYear.toString()[1] + p0.split("/").last);
                          int enteredMonth = int.parse(p0.split("/").first);
                          DateTime date = DateTime(enteredYear, enteredMonth + 1);

                          if (date.isBefore(DateTime.now())) {
                            return "Please enter valid expiry.".tr;
                          } else {
                            return null;
                          }
                        }
                      },
                      controller: controller.expiryController,
                      labelText: "Expiry / Validity".tr,
                      themeColor: widget.themeColor ?? primaryColor,
                      maxlength: 5),
                ),
              ),
            ),
            const SizedBox(width: 15),
            Obx(() => Expanded(
                  child: Focus(
                    onFocusChange: (hasFocus) {
                      if (controller.cvvController.text != "") {
                        controller.cvvForm.currentState?.validate();
                      }
                    },
                    child: Form(
                      key: controller.cvvForm,
                      child: CommonWidgets.commonUnderLineTextField(
                          isObscure: true,
                          inputFormatters: [FilteringTextInputFormatter.allow(RegExp('[0-9]'))],
                          keyboardType: const TextInputType.numberWithOptions(decimal: false, signed: false),
                          validator: (p0) {
                            if (p0!.isEmpty) {
                              return "Please enter CVV".tr;
                            } else if (p0.length < (controller.cardType.value == "Amex" ? 4 : 3)) {
                              return "Please enter valid CVV".tr;
                            } else {
                              return null;
                            }
                          },
                          controller: controller.cvvController,
                          maxlength: controller.cardType.value == "Amex" ? 4 : 3,
                          labelText: "CVV",
                          suffixIcon: widget.packageMode == PackageMode.debug
                              ? Container(height: 20, width: 20, color: widget.themeColor)
                              : InkWell(
                                  onTap: () {
                                    cvvDialog();
                                  },
                                  child: SvgPicture.asset(AssetPath.cvvIconSVG, package: 'sadad_payment_flutter', color: widget.themeColor)),
                          themeColor: widget.themeColor ?? primaryColor),
                    ),
                  ),
                )),
          ],
        ),

        // Padding(
        //   padding: const EdgeInsets.symmetric(vertical: 10),
        //   child: Row(
        //     children: [
        //       SizedBox(
        //           height: 15,
        //           width: 15,
        //           child: Checkbox(
        //             value: controller.rememberMe,
        //             onChanged: (value) {
        //               controller.rememberMe = value!;
        //               controller.update();
        //             },
        //             activeColor: AppColors.green,
        //           )),
        //       const SizedBox(width: 8),
        //       Text(
        //         "Save this card for faster checkout for next time",
        //         style: TextStyle(fontSize: 10, color: AppColors.black.withOpacity(0.5)),
        //       )
        //     ],
        //   ),
        // ),
        const SizedBox(height: 12),
        paymentButton(
          type: "credit",
          title: "Pay".tr + " ${widget.amount.toStringAsFixed(2)}",
          onTap: () async {
            if (controller.expiryValidityForm.currentState!.validate()) {}
            if (controller.creditCardHolderForm.currentState!.validate()) {}
            if (controller.creditCardNumberForm.currentState!.validate()) {}
            if (controller.cvvForm.currentState!.validate()) {}

            if (controller.expiryValidityForm.currentState!.validate() &&
                controller.creditCardHolderForm.currentState!.validate() &&
                controller.creditCardNumberForm.currentState!.validate() &&
                controller.cvvForm.currentState!.validate()) {
              String cardType = controller.getCardType(controller.cardNumberController.text);
              String firstSixDigit = controller.cardNumberController.text.replaceAll(" ", "").substring(0, 6);
              if (cardType == "Amex" && is_american_express == "0") {
                Get.snackbar("Sadad Pay".tr, "American Express card not allowed for your merchant.".tr);
                // DispatchQueue.main.async {
                // let sb = AP_Snackbar()
                // sb.createWithAction(text: "American Express card not allowed for your merchant.",actionTitle: "Ok",action: {
                // sb.sbLenght = .unHide
                // sb.hide()
                // })
                // }
              } else {
                CheckedAllowedCountryModel? data = await AppServices.checkAllowedCountry(cardsixdigit: firstSixDigit, token: widget.token);

                ///change on live
                if (data?.isAllowed == true) {
                  var response = await AppServices.GenerateCreditCardTransectionID(
                      hash: checkSum,
                      card_type: cardType,
                      token: widget.token,
                      cardsixdigit: firstSixDigit,
                      mobileNumber: widget.mobile,
                      txnAmount: widget.amount,
                      productDetail: widget.productDetail,
                      is_american_express: is_american_express == "0" ? false : true,
                      is_cybersourse_visa: int.parse(is_cybersourse_visa),
                      credit_card_bankpage_type: credit_card_bankpage_type,
                      is_cybersourse_mastercard: int.parse(is_cybersourse_mastercard),
                      userMetaPreference: userMetaPreference!);

                  if (response is String) {
                    AppDialog.commonWarningDialog(
                        context: context,
                        title: "error",
                        subTitle: response,
                        buttonOnTap: () {
                          Navigator.pop(context);
                          Navigator.pop(context);
                        },
                        buttonText: "Okay".tr);
                  } else if (response is TransactionIdDetailsModel) {
                    // String sadad_id = response.sadadId ?? "";
                    //String invoiceNumber = response.invoicenumber ?? "";
                    WebViewDetailsModel webViewDetailsModel = WebViewDetailsModel(
                        packageMode: widget.packageMode ?? PackageMode.release,
                        token: widget.token,
                        isAmexEnableForAdmin: is_american_express == "0" ? true : false,
                        isMasterEnableForCyber: userMetaPreference?.isallowedtocybersourcemastercard ?? false ? 0 : 1,
                        isVisaEnableForCyber: userMetaPreference?.isallowedtocybersourcevisa ?? false ? 0 : 1,
                        isMasterEnableForCyberAdmin: int.parse(is_cybersourse_mastercard),
                        isVisaEnableForCyberAdmin: int.parse(is_cybersourse_visa),
                        creditcardType: cardType,
                        sadadId: response.sadadId ?? "",
                        cardnumber: controller.cardNumberController.text,
                        expiryDate: controller.expiryController.text,
                        cvv: controller.cvvController.text,
                        cardHolderName: controller.cardHolderNameController.text,
                        customerName: widget.customerName,
                        transactionAmount: (widget.amount * 100),
                        transactionId: response.invoicenumber.toString(),
                        contactNumber: widget.mobile,
                        paymentMethod: "credit",
                        email: widget.email);
                    Navigator.push(context, MaterialPageRoute(
                      builder: (context) {
                        return PaymentWebViewScreen(webViewDetailsModel: webViewDetailsModel);
                      },
                    )).then((value) {
                      Navigator.pop(context, value);
                    });
                  }
                } else {
                  AppDialog.commonWarningDialog(
                      context: context,
                      title: "Not Allowed".tr,
                      subTitle: "Transaction is not allowed from your country.".tr,
                      buttonOnTap: () {
                        Navigator.pop(context);
                        Navigator.pop(context);
                      },
                      buttonText: "Okay".tr);
                }
                print(data);
              }
            }
            setState(() {});
          },
        ),
        const SizedBox(height: 15),
        commonSecurityText()
      ]),
    );
  }

  Widget sadadPayContainer() {
    return Container(
      margin: const EdgeInsets.all(10),
      padding: const EdgeInsets.all(20),
      decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(5),
          color: Colors.white,
          border: Border.all(color: AppColors.borderColor),
          boxShadow: const [BoxShadow(color: AppColors.black12, offset: Offset(0, 0), blurRadius: 5, blurStyle: BlurStyle.outer)]),
      child: Form(
        key: controller.sadadPayForm,
        child: Column(children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Image.asset(
                    AssetPath.sadadLogo,
                    package: 'sadad_payment_flutter',
                    width: 50,
                  ),
                  const SizedBox(height: 5),
                  Text("Login with your Sadad account".tr, style: TextStyle(fontSize: 10, color: AppColors.black.withOpacity(0.5)))
                ],
              ),
              // Column(
              //   crossAxisAlignment: CrossAxisAlignment.end,
              //   children: [
              //     Image.asset(
              //       AssetPath.sadadLogo,
              //       package: 'sadad_payment_flutter',
              //       width: 50,
              //     ),
              //     SizedBox(height: 5),
              //     Text("Or Scan", style: TextStyle(fontSize: 10, color: AppColors.black.withOpacity(0.5)))
              //   ],
              // ),
            ],
          ),
          const SizedBox(height: 15),
          IgnorePointer(
            ignoring: controller.sadadPayOTPSent,
            child: Column(
              children: [
                CommonWidgets.commonUnderLineTextField(
                    helperText: null,
                    validator: (p0) {
                      if (p0!.isEmpty) {
                        return "Please enter cell number.".tr;
                      } else if (p0.length < 8) {
                        return "Please enter valid cell number".tr;
                      } else {
                        return null;
                      }
                    },
                    maxlength: 8,
                    inputFormatters: [FilteringTextInputFormatter.allow(RegExp('[0-9]'))],
                    keyboardType: const TextInputType.numberWithOptions(signed: false, decimal: false),
                    readOnly: controller.sadadPayOTPSent,
                    controller: controller.sadadPayCellNumberController,
                    themeColor: widget.themeColor ?? primaryColor,
                    labelText: "Cell Number".tr),
                CommonWidgets.commonUnderLineTextField(
                    helperText: null,
                    validator: (p0) {
                      if (p0!.isEmpty) {
                        return "Please enter password".tr;
                      } else {
                        return null;
                      }
                    },
                    readOnly: controller.sadadPayOTPSent,
                    controller: controller.sadadPayPasswordController,
                    isObscure: controller.isObscure,
                    themeColor: widget.themeColor ?? primaryColor,
                    labelText: "Password".tr,
                    suffixIcon: Padding(
                      padding: const EdgeInsets.all(12),
                      child: GestureDetector(
                        onTap: () {
                          if (controller.isObscure) {
                            controller.isObscure = false;
                          } else {
                            controller.isObscure = true;
                          }
                          controller.update();
                        },
                        child: widget.packageMode == PackageMode.debug
                            ? Container(height: 20, width: 20, color: widget.themeColor)
                            : Image.asset(controller.isObscure ? AssetPath.icEyeClose : AssetPath.icEyeOpen, package: 'sadad_payment_flutter', color: widget.themeColor, height: 20),
                      ),
                    )),
              ],
            ),
          ),
          const SizedBox(height: 35),
          paymentButton(
            type: "sadadPay",
            title: "Pay".tr + " ${widget.amount.toStringAsFixed(2)}",
            onTap: () async {
              if (controller.sadadPayForm.currentState!.validate()) {
                AppDialog.showProcess(context, widget.themeColor ?? primaryColor);
                controller.sendOtpModel =
                    await AppServices.sadadPayLogin(token: widget.token, password: controller.sadadPayPasswordController.text, cellnumber: controller.sadadPayCellNumberController.text);
                Navigator.pop(context);
                if (controller.sendOtpModel != null) {
                  controller.sadadPayOTPSent = true;
                  controller.counter.value = 59;
                  controller.initOTPCounter();
                  otpVerificationDialog();
                  controller.update();
                } else {
                  AppDialog.commonWarningDialog(
                      context: context,
                      title: "Issue".tr,
                      subTitle: "We are not able to proceed please try again.".tr,
                      buttonOnTap: () {
                        Navigator.pop(context);
                        Navigator.pop(context);
                      },
                      buttonText: "Okay");
                }
              }

              // var amount = widget.amount.toString();
              // var values = amount.split('.');
              // if (values.length > 1) {
              //   if (values[1].length > 2) {
              //     showDialog(
              //         context: context,
              //         builder: (context) {
              //           return AlertDialog(
              //             actions: [
              //               InkWell(
              //                   onTap: () {
              //                     Navigator.pop(context);
              //                   },
              //                   child: Text("Okay".tr))
              //             ],
              //             title: Text("Please enter amount upto 2 decimal points".tr),
              //           );
              //         });
              //   } else {
              //     SadadPayMinAmountCheckModel? sadadPayMinAmountCheckModel = await AppServices.sadadPayMinAmountCheck(token: widget.token);
              //     if (sadadPayMinAmountCheckModel == null) {
              //       showDialog(
              //           context: context,
              //           builder: (context) {
              //             return AlertDialog(
              //               actions: [
              //                 InkWell(
              //                     onTap: () {
              //                       Navigator.pop(context);
              //                     },
              //                     child: Text("Okay".tr))
              //               ],
              //               title: Text("Something went wrong. Please try again".tr),
              //             );
              //           });
              //     } else {
              //       double value = double.parse(sadadPayMinAmountCheckModel.value);
              //       if (AppConstant.transactionAmount < value) {
              //         showDialog(
              //             context: context,
              //             builder: (context) {
              //               return AlertDialog(
              //                 actions: [
              //                   InkWell(
              //                       onTap: () {
              //                         Navigator.pop(context);
              //                       },
              //                       child: Text("Okay".tr))
              //                 ],
              //                 title: Text("The Given Transaction Amount is less than ".tr + "${value}." + " Please try with valid amount!".tr),
              //               );
              //             });
              //       } else {
              //         WebViewDetailsModel webViewDetailsModel = WebViewDetailsModel(
              //             packageMode: widget.packageMode ?? PackageMode.release,
              //             token: widget.token,
              //             isAmexEnableForAdmin: is_american_express == "0" ? true : false,
              //             isMasterEnableForCyber: userMetaPreference?.isallowedtocybersourcemastercard ?? false ? 0 : 1,
              //             isVisaEnableForCyber: userMetaPreference?.isallowedtocybersourcevisa ?? false ? 0 : 1,
              //             isMasterEnableForCyberAdmin: int.parse(is_cybersourse_mastercard),
              //             isVisaEnableForCyberAdmin: int.parse(is_cybersourse_visa),
              //             creditcardType: "",
              //             sadadId: "",
              //             cardnumber: controller.cardNumberController.text,
              //             expiryDate: controller.expiryController.text,
              //             cvv: controller.cvvController.text,
              //             cardHolderName: controller.cardHolderNameController.text,
              //             customerName: widget.customerName,
              //             transactionAmount: (widget.amount * 100),
              //             transactionId: "",
              //             contactNumber: widget.mobile,
              //             productDetail: widget.productDetail,
              //             paymentMethod: "sadadPay",
              //             email: widget.email);
              //         Navigator.push(context, MaterialPageRoute(
              //           builder: (context) {
              //             return PaymentWebViewScreen(webViewDetailsModel: webViewDetailsModel);
              //           },
              //         )).then((value) {
              //           Navigator.pop(context, value);
              //         });
              //       }
              //     }
              //   }
              // }
            },
          ),
          const SizedBox(height: 15),
          commonSecurityText()
        ]),
      ),
    );
  }

  Widget debitCardContainer() {
    return Container(
      height: 370,
      margin: const EdgeInsets.all(10),
      padding: const EdgeInsets.only(top: 5, right: 20, left: 20, bottom: 20),
      decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(5),
          color: Colors.white,
          border: Border.all(color: AppColors.borderColor),
          boxShadow: const [BoxShadow(color: AppColors.black12, offset: Offset(0, 0), blurRadius: 5, blurStyle: BlurStyle.outer)]),
      child: Column(children: [
        Row(
          children: [
            Image.asset(AssetPath.debitNaps, package: 'sadad_payment_flutter', width: 85),
            const SizedBox(width: 16),
            Image.asset(AssetPath.debitHimyan, package: 'sadad_payment_flutter', width: 85),
            // Text(
            //   "Debit Card".tr,
            //   style: TextStyle(color: primaryColor, fontSize: 15, fontWeight: FontWeight.bold),
            // ),
          ],
        ),
        const SizedBox(
          height: 13,
        ),
        // const SizedBox(height: 15),
        Text(
          "When you choose to complete your payment using a debit card issued in Qatar, you will be temporarily redirected to the QPay website. Once the transaction is completed, you'll be taken back to the Sadad to view your confirmation."
              .tr,
          style: TextStyle(fontFamily: "openSans", fontSize: 12, fontWeight: FontWeight.w400, letterSpacing: 0.5, color: AppColors.black.withOpacity(0.7), height: 1.7),
        ),
        const Spacer(),
        paymentButton(
          type: "Debit",
          title: "Pay".tr + " ${widget.amount.toStringAsFixed(2)}",
          onTap: () async {
            var response = await AppServices.GenerateDebitCardTransactionID(
                hash: checkSum, token: widget.token, mobileNumber: widget.mobile, txnAmount: widget.amount, productDetail: widget.productDetail, userMetaPreference: userMetaPreference!);
            if (response is int) {
              showDialog(
                  context: context,
                  builder: (context) {
                    return Dialog(
                      child: Text(response.toString()),
                    );
                  });
            } else if (response is TransactionIdDetailsModel) {
              WebViewDetailsModel webViewDetailsModel = WebViewDetailsModel(
                  packageMode: widget.packageMode ?? PackageMode.release,
                  token: widget.token,
                  isAmexEnableForAdmin: is_american_express == "0" ? true : false,
                  isMasterEnableForCyber: userMetaPreference?.isallowedtocybersourcemastercard ?? false ? 0 : 1,
                  isVisaEnableForCyber: userMetaPreference?.isallowedtocybersourcevisa ?? false ? 0 : 1,
                  isMasterEnableForCyberAdmin: int.parse(is_cybersourse_mastercard),
                  isVisaEnableForCyberAdmin: int.parse(is_cybersourse_visa),
                  creditcardType: "",
                  sadadId: response.sadadId ?? "",
                  cardnumber: controller.cardNumberController.text,
                  expiryDate: controller.expiryController.text,
                  cvv: controller.cvvController.text,
                  cardHolderName: controller.cardHolderNameController.text,
                  customerName: widget.customerName,
                  transactionAmount: (widget.amount * 100),
                  transactionId: response.invoicenumber.toString(),
                  contactNumber: widget.mobile,
                  paymentMethod: "debit",
                  email: widget.email);
              Navigator.push(context, MaterialPageRoute(
                builder: (context) {
                  return PaymentWebViewScreen(webViewDetailsModel: webViewDetailsModel);
                },
              )).then((value) {
                Navigator.pop(context, value);
              });
            }
          },
        ),
        const SizedBox(height: 15),
        commonSecurityText()
      ]),
    );
  }

  bottomContainer() {
    return Padding(
      padding: const EdgeInsets.symmetric(horizontal: 8.0),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Row(
              mainAxisAlignment: MainAxisAlignment.spaceAround,
              children: [securityContainer(AssetPath.mcAfee), securityContainer(AssetPath.comodoSecure), securityContainer(AssetPath.symantec), securityContainer(AssetPath.pciDss)]),
          const SizedBox(height: 15),
          Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              ImageIcon(const AssetImage(AssetPath.lock, package: 'sadad_payment_flutter'), size: 10, color: widget.themeColor),
              const SizedBox(width: 3),
              Text("100% secured Payment ".tr, style: const TextStyle(fontSize: 10, color: AppColors.greyTextColor)),
              Text(
                "Powered by Sadad".tr,
                style: TextStyle(fontSize: 10, color: primaryColor),
              )
            ],
          ),
          const SizedBox(height: 15)
        ],
      ),
    );
  }

  commonSecurityText() {
    return Row(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        widget.packageMode == PackageMode.debug
            ? Container(height: 20, width: 20, color: widget.themeColor)
            : ImageIcon(const AssetImage(AssetPath.lock, package: 'sadad_payment_flutter'), size: 10, color: widget.themeColor),
        const SizedBox(width: 3),
        Expanded(
          child: Wrap(children: [
            Text("Payment is Secured with 256bit SSL encryption ".tr, style: const TextStyle(fontSize: 10, color: AppColors.greyTextColor)),
            Text(
              "(You are safe)".tr,
              style: TextStyle(fontSize: 10, color: primaryColor),
            )
          ]),
        )
      ],
    );
  }

  Widget securityContainer(String imagepath) {
    return Container(
      height: 50,
      padding: const EdgeInsets.all(5),
      decoration: BoxDecoration(color: AppColors.borderColor, borderRadius: BorderRadius.circular(5)),
      child: widget.packageMode == PackageMode.debug ? Container(height: 20, width: 20, color: widget.themeColor) : Image.asset(imagepath, width: Get.width * 0.20, package: 'sadad_payment_flutter'),
    );
  }

  Widget paymentButton({required String title, required Function() onTap, required String type}) {
    return Directionality(
      textDirection: TextDirection.ltr,
      child: InkWell(
          onTap: onTap,
          child: Container(
            decoration: BoxDecoration(color: widget.paymentButtonColor ?? primaryColor, borderRadius: BorderRadius.circular(10), border: Border.all(color: AppColors.paymentButtonBorder)),
            height: 50,
            child: Center(child: priceWidget(price: title)),
          )),
    );
  }

  Future<void> creditCardSettings({required String token}) async {
    Future.delayed(Duration.zero, () {
      AppDialog.showProcess(context, widget.themeColor ?? primaryColor);
    });
    String? ipAddress = await NetworkInfo().getWifiIPv6();
    print(ipAddress);

    creditCardSettingsModel = await AppServices.getCreditCardSetting(token: token, context: context);
    creditCardSettingsModel.forEach((element) {
      if (element.key == "credit_card_bankpage_type") {
        credit_card_bankpage_type = element.value ?? "";
      } else if (element.key == 'is_american_express') {
        is_american_express = element.value ?? "";
      } else if (element.key == 'is_cybersourse_visa') {
        is_cybersourse_visa = element.value ?? "";
      } else if (element.key == 'is_cybersourse_mastercard') {
        is_cybersourse_mastercard = element.value ?? "";
      }
    });
    userMetaPreference = await AppServices.getUserPreferences(token: token);
    checkSum = await AppServices.generateChecksum(token: token, mobile: widget.mobile, amount: widget.amount, product_detail: widget.productDetail) ?? "";
    Navigator.pop(context);
    if (creditCardSettingsModel.isEmpty || userMetaPreference == null || checkSum.isEmpty) {
      AppDialog.commonWarningDialog(
          context: context,
          title: "Something went wrong".tr,
          subTitle: "",
          buttonOnTap: () {
            Navigator.pop(context);
            Navigator.pop(context);
          },
          buttonText: "Okay".tr);
    }
  }

  cvvDialog() {
    showDialog(
      barrierColor: AppColors.white.withOpacity(0.86),
      barrierDismissible: false,
      context: context,
      builder: (context) {
        return Dialog(
          alignment: Alignment.center,
          backgroundColor: Colors.transparent,
          elevation: 0,
          child: Padding(
            padding: const EdgeInsets.all(14),
            child: Container(
              height: 270,
              padding: const EdgeInsets.all(15),
              child: Stack(
                alignment: Alignment.center,
                fit: StackFit.loose,
                children: [
                  Container(
                    height: 270,
                    margin: const EdgeInsets.all(10),
                    padding: const EdgeInsets.all(15),
                    decoration: BoxDecoration(color: AppColors.white, borderRadius: BorderRadius.circular(5), boxShadow: [
                      BoxShadow(
                        color: AppColors.grey.withOpacity(0.5),
                        blurRadius: 40,
                        spreadRadius: 8,
                      )
                    ]),
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.center,
                      mainAxisSize: MainAxisSize.min,
                      children: [
                        Image.asset(AssetPath.cvvCard, package: 'sadad_payment_flutter', width: MediaQuery.of(context).size.width * 0.45),
                        const SizedBox(height: 15),
                        Text("What is CVV".tr, style: const TextStyle(fontSize: 12, fontWeight: FontWeight.w700)),
                        const SizedBox(height: 15),
                        Text(
                          "CVV is the three digit number on the back of your credit/debit card.".tr,
                          textAlign: TextAlign.center,
                          style: TextStyle(fontSize: 12, color: AppColors.black.withOpacity(0.5)),
                        ),
                      ],
                    ),
                  ),
                  Align(
                      alignment: Alignment.topRight,
                      child: InkWell(
                          onTap: () {
                            Navigator.pop(context);
                          },
                          child: Container(
                              padding: const EdgeInsets.all(3),
                              decoration: const BoxDecoration(color: AppColors.white, shape: BoxShape.circle),
                              child: const Icon(
                                Icons.close,
                                color: AppColors.iconColor,
                                size: 18,
                              ))))
                ],
              ),
            ),
          ),
        );
      },
    );
  }

  otpVerificationDialog() {
    showDialog(
      barrierColor: AppColors.white.withOpacity(0.95),
      barrierDismissible: false,
      context: context,
      builder: (context) {
        return Dialog(
          alignment: Alignment.center,
          backgroundColor: Colors.transparent,
          elevation: 0,
          child: SizedBox(
            height: 250,
            child: Stack(
              children: [
                Container(
                  margin: const EdgeInsets.all(20),
                  height: 250,
                  decoration: BoxDecoration(color: AppColors.white, borderRadius: BorderRadius.circular(10), boxShadow: [
                    BoxShadow(
                      color: AppColors.grey.withOpacity(0.5),
                      blurRadius: 20,
                      spreadRadius: 4,
                    )
                  ]),
                  padding: const EdgeInsets.all(20),
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    mainAxisSize: MainAxisSize.min,
                    children: [
                      Padding(
                        padding: const EdgeInsets.all(20),
                        child: Text("OTP Verification".tr, style: const TextStyle(color: AppColors.black, fontWeight: FontWeight.bold, fontSize: 20)),
                      ),
                      PinFieldAutoFill(
                        controller: controller.sadadPayOTPController,
                        textInputAction: TextInputAction.go,
                        inputFormatters: [
                          FilteringTextInputFormatter.allow(RegExp(r"[0-9]")),
                        ],
                        keyboardType: const TextInputType.numberWithOptions(signed: false, decimal: true),
                        decoration: BoxLooseDecoration(
                          radius: const Radius.circular(4),
                          hintText: '------',
                          strokeColorBuilder: FixedColorBuilder(Colors.black.withOpacity(0.3)),
                        ),
                        codeLength: 6,
                        onCodeSubmitted: (code) async {},
                        onCodeChanged: (code) async {
                          print(code!.length);
                          if (code.length == 6) {
                            AppDialog.showProcess(context, widget.themeColor ?? primaryColor);
                            bool? success = await AppServices.sadadPayVerifyOtp(token: widget.token, otp: code, userId: controller.sendOtpModel!.userId!.toString());
                            Navigator.pop(context);
                            if (success!) {
                              String? ipAddress = await NetworkInfo().getWifiIPv6();
                              Map? data = await AppServices.sadadPayTransactions(
                                  ipAddress: ipAddress ?? "",
                                  sadadId: controller.sendOtpModel!.id!,
                                  userId: controller.sendOtpModel!.userId!,
                                  token: widget.token,
                                  amount: widget.amount,
                                  productDetails: widget.productDetail);
                              if (data == null) {
                                AppDialog.commonWarningDialog(
                                    context: context,
                                    title: "Issue".tr,
                                    subTitle: "We are not able to proceed please try again.".tr,
                                    buttonOnTap: () {
                                      Navigator.pop(context);
                                      Navigator.pop(context);
                                    },
                                    buttonText: "Okay");
                              } else {
                                Navigator.pop(context);
                                Navigator.pop(context, data);
                              }
                            }
                          }
                        },
                      ),
                      const SizedBox(height: 20),
                      Obx(() {
                        return RichText(
                            text: TextSpan(children: [
                          TextSpan(text: controller.counter.value > 0 ? "OTP is valid Upto ".tr : "Can't receive the OTP? ".tr, style: const TextStyle(color: AppColors.black, fontSize: 13)),
                          TextSpan(
                              recognizer: TapGestureRecognizer()
                                ..onTap = () async {
                                  if (controller.counter.value < 1) {
                                    if (controller.counter.value == 0) {
                                      AppDialog.showProcess(context, widget.themeColor ?? primaryColor);
                                      bool success = await AppServices.sadadPayResendOtp(token: controller.sendOtpModel!.id!);
                                      Navigator.pop(context);
                                      if (success) {
                                        controller.counter.value = 59;
                                        controller.initOTPCounter();
                                        controller.sadadPayOTPController.clear();
                                      } else {
                                        AppDialog.commonWarningDialog(
                                            context: context,
                                            title: "Issue".tr,
                                            subTitle: "We are not able to proceed please try again.".tr,
                                            buttonOnTap: () {
                                              Navigator.pop(context);
                                              Navigator.pop(context);
                                              Navigator.pop(context);
                                            },
                                            buttonText: "Okay".tr);
                                      }
                                    }
                                  }
                                },
                              text: controller.counter.value > 0 ? controller.counter.value.toString() : "Resend".tr,
                              style: TextStyle(color: widget.themeColor, fontSize: 13, fontWeight: FontWeight.w600)),
                          if (controller.counter.value > 0) TextSpan(text: " seconds".tr, style: const TextStyle(color: AppColors.black, fontSize: 13)),
                        ]));
                      }),
                      const SizedBox(height: 20),
                    ],
                  ),
                ),
                Align(
                    alignment: Alignment.topRight,
                    child: GestureDetector(
                      onTap: () {
                        controller.sadadPayOTPSent = false;
                        controller.counter.value = 59;
                        controller.timer!.cancel();
                        Navigator.pop(context);
                        controller.update();
                      },
                      child: Container(
                        height: 60,
                        width: 60,
                        child: Image.asset(AssetPath.icClose, package: "sadad_payment_flutter", fit: BoxFit.fill),
                      ),
                    ))
              ],
            ),
          ),
        );
      },
    );
  }

  void setLanguage() {
    Get.addTranslations({"en": en, "ar": ar});
    Get.updateLocale(selectedLanguage);
  }
}

enum PaymentType { creditCard, debitCard, sadadPay, wallet }

enum PackageMode { release, debug }
