import 'dart:convert';
import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:get/get_core/src/get_main.dart';
import 'package:http/http.dart' as http;
import 'package:sadad_payment_flutter/apputils/appcolors.dart';
import 'package:sadad_payment_flutter/apputils/appdialogs.dart';
import 'package:sadad_payment_flutter/apputils/appstrings.dart';
import 'package:sadad_payment_flutter/model/checkedallowedcountrymodel.dart';
import 'package:sadad_payment_flutter/model/creditcardsettingsmodel.dart';
import 'package:sadad_payment_flutter/model/sadadpayminimumamountcheckmodel.dart';
import 'package:sadad_payment_flutter/model/sendOtpModel.dart';
import 'package:sadad_payment_flutter/model/transactionIdDetailsModel.dart';
import 'package:sadad_payment_flutter/model/usermetapreference.dart';

class Baseurl {
  static const baseUrl = "";
}

class ApiEndPoint {
  static const primaryColor = "";
  static const live_CreditCardSettings =
      "https://mapi.sadadqa.com/sdk-api/settings?filter[where][or][0][key]=ENABLE_CREDIT_CARD&filter[where][or][1][key]=ENABLE_DEBIT_CARD&filter[where][or][2][key]=credit_card_bankpage_type&filter[where][or][3][key]=is_apple_pay_payment&filter[where][or][4][key]=is_google_pay_payment&filter[where][or][5][key]=is_american_express&filter[where][or][6][key]=is_cybersourse_visa&filter[where][or][7][key]=is_cybersourse_mastercard";
  static const live_CheckCountry = "https://mapi.sadadqa.com/sdk-api/usermetapreferences/checkAllowedCountry";
  static const LIVE_usermeta = "https://mapi.sadadqa.com/sdk-api/usermetapreferences";
  static const LIVE_TransectionURL = "https://mapi.sadadqa.com/sdk-api/transactions/genchecksum";
  static const LIVE_generateTransaction = "https://mapi.sadadqa.com/sdk-api/transactions";
  static const LIVE_amex_creditCardURL = "https://sadadqa.com/amexpayment";
  static const LIVE_migs_pay_creditCardURL = "https://sadadqa.com/ccapi/pay.php";
  static const LIVE_mpgs_pay_creditCardURL = "https://sadadqa.com/Sadadprocessing/pay.php";
  static const LIVE_PatchTransactionURL = "https://mapi.sadadqa.com/sdk-api/transactions/sdkpay";
  static const LIVE_debitCardURL = "https://sadadqa.com/napsapi/pay.php";
  static const LIVE_minAmount = "https://mapi.sadadqa.com/sdk-api/settings?filter[where][key]=sdk_min_amount";
  static const LIVE_SDKLoginURL = "https://staging.sadadqa.com/login";
  static const LIVE_applepay_URL = "https://sadadqa.com/sdk/processapplepay";

  //============================================================
  //=========== Dev Server URL =================================
  static const test_CreditCardSettings =
      "http://176.58.99.102:3003/sdk-api/settings?filter[where][or][0][key]=ENABLE_CREDIT_CARD&filter[where][or][1][key]=ENABLE_DEBIT_CARD&filter[where][or][2][key]=credit_card_bankpage_type&filter[where][or][3][key]=is_apple_pay_payment&filter[where][or][4][key]=is_google_pay_payment&filter[where][or][5][key]=is_american_express&filter[where][or][6][key]=is_cybersourse_visa&filter[where][or][7][key]=is_cybersourse_mastercard";
  static const test_CheckCountry = "http://176.58.99.102:3003/sdk-api/usermetapreferences/checkAllowedCountry";
  static const test_usermeta = "http://176.58.99.102:3003/sdk-api/usermetapreferences";
  static const test_TransectionURL = "http://176.58.99.102:3003/sdk-api/transactions/genchecksum";
  static const test_generateTransaction = "http://176.58.99.102:3003/sdk-api/transactions";
  static const test_amex_creditCardURL = "https://sadad.de/sadadinvoice/amexpayment";
  static const test_migs_pay_creditCardURL = "https://sadad.de/sadadinvoice/ccapi/pay.php";
  static const test_mpgs_pay_creditCardURL = "https://sadad.de/sadadinvoice/Sadadprocessing/pay.php";
  static const test_PatchTransactionURL = "http://176.58.99.102:3003/sdk-api/transactions/sdkpay";

  //static const test_PatchTransactionURL = "https://msandboxapi.sadadqa.com/sdk-api/transactions/sdkpay";
  static const test_debitCardURL = "https://sadad.de/sadadinvoice/napsapi/pay.php";
  static const test_minAmount = "http://176.58.99.102:3003/sdk-api/settings?filter[where][key]=sdk_min_amount";
  static const test_SDKLoginURL = "https://sadadsdk.sadadqa.com/login";
  static const test_applepay_URL = "https://sadad.de/sadadinvoice/sdk/processapplepay";
  static const test_sadadPayLogin = "http://176.58.99.102:3003/sdk-api/users/login";
  static const test_verifyOtp = "http://176.58.99.102:3003/sdk-api/usermetaauths/verify";
  static const test_resendOtp = "http://176.58.99.102:3003/sdk-api/usermetaauths/resendOtp?type=login";
  static const test_transaction = "http://sadad.de:3003/sdk-api/transactions";
}

class AppServices {
  static Future<void> getAccessToken() async {
    final url = Uri.parse(
      'https://sadad.de/sadadSDKTestConfig/index.php',
    );
    Map<String, String> header = {'Content-Type': 'application/json'};
    var result = await http.post(
      url,
      headers: header,
    );

    if (result.statusCode == 200) {
      var token = '${jsonDecode(result.body)}';
      print('notification count:::${token}');
    }
  }

  // let request = NSMutableURLRequest(url: NSURL(string: "\(Constant.creditcard_Settings)")! as URL)
  // request.httpMethod = "GET"
  //
  // request.setValue("application/json", forHTTPHeaderField: "Content-Type")
  // request.setValue("\(strAccessToken)", forHTTPHeaderField: "Authorization")

  static Future<List<CreditCardSettingsModel>> getCreditCardSetting({required String token, required BuildContext context}) async {
    final url = Uri.parse(
      ApiEndPoint.test_CreditCardSettings,
    );
    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};
    var result = await http.get(
      url,
      headers: header,
    );
    List<CreditCardSettingsModel> creditCardSettings = [];
    if (result.statusCode == 200) {
      List response = jsonDecode(result.body);
      creditCardSettings = response.map((e) => CreditCardSettingsModel.fromJson(e)).toList();
      print('notification count:::${response}');
    }
    return creditCardSettings;
  }

  static Future<UserMetaPreference?> getUserPreferences({required String token}) async {
    final url = Uri.parse(
      ApiEndPoint.test_usermeta,
    );
    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};
    var result = await http.get(
      url,
      headers: header,
    );

    if (result.statusCode == 200) {
      List response = jsonDecode(result.body);
      if (response.isNotEmpty) {
        UserMetaPreference userMetaPreference = UserMetaPreference.fromJson(response.first);
        return userMetaPreference;
      } else {
        return null;
      }
    }
    return null;
  }

  static Future<CheckedAllowedCountryModel?> checkAllowedCountry({required String token, required String cardsixdigit}) async {
    final url = Uri.parse(
      ApiEndPoint.test_CheckCountry,
    );
    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};
    Map<String, String> body = {"cardsixdigit": cardsixdigit};
    CheckedAllowedCountryModel model = CheckedAllowedCountryModel(isAllowed: false, isDebitCard: false);
    var result = await http.post(
      url,
      headers: header,
      body: json.encode(body),
    );

    if (result.statusCode == 200) {
      var response = jsonDecode(result.body);
      model = CheckedAllowedCountryModel.fromJson(response);
      return model;
    } else {
      var response = jsonDecode(result.body);
      print(response);
    }
    return model;
  }

  static Future<dynamic> GenerateCreditCardTransectionID(
      {required String card_type,
      required String token,
      required String cardsixdigit,
      required String mobileNumber,
      required double txnAmount,
      required List<Map> productDetail,
      required bool is_american_express,
      required int is_cybersourse_mastercard,
      required String credit_card_bankpage_type,
      required String hash,
      required UserMetaPreference userMetaPreference,
      required int is_cybersourse_visa}) async {
    final url = Uri.parse(
      ApiEndPoint.test_generateTransaction,
    );
    int TypeOfCard = 0;
    if (card_type == "Visa") {
      TypeOfCard = 1;
    } else if (card_type == "Mastercard") {
      TypeOfCard = 2;
    } else if (card_type == "Amex") {
      TypeOfCard = 3;
    }

    String cardID = "0";
    if (card_type == "Amex" && is_american_express) {
      cardID = "0";
    } else if (credit_card_bankpage_type == "3") {
      if (card_type == "Mastercard" && (userMetaPreference.isallowedtocybersourcemastercard == 0 || is_cybersourse_mastercard == 0)) {
        cardID = "2";
        credit_card_bankpage_type = "2";
      } else if (card_type == "Visa" && (userMetaPreference.isallowedtocybersourcevisa == 0 || is_cybersourse_visa == 0)) {
        cardID = "2";
        credit_card_bankpage_type = "2";
      } else {
        cardID = credit_card_bankpage_type;
      }
    } else {
      cardID = credit_card_bankpage_type;
    }
    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};
    Map transaction_summary = {
      "MOBILE_NO": mobileNumber,
      "TXN_AMOUNT": txnAmount,
      "PRODUCT_DETAILS": productDetail,
    };
    Map body = {
      "amount": txnAmount,
      "transactionentityId": AppConstant.transection_SDK_ID,
      "transactionmodeId": AppConstant.CreditCard_TransactionMode_ID,
      "transactionstatusId": AppConstant.TransectionInProgress,
      "sourceofTxn": AppConstant.TransectionSource,
      "transaction_summary": transaction_summary,
      "cardschemeid": TypeOfCard,
      "creditcardpaymentmodeid": int.parse(cardID),
      "hash": hash
    };
    print(url);
    var result = await http.post(url, headers: header, body: jsonEncode(body));

    if (result.statusCode == 200) {
      var response = jsonDecode(result.body);
      TransactionIdDetailsModel transactionIdDetailsModel = TransactionIdDetailsModel.fromJson(response);
      print('notification count:::${response}');
      return transactionIdDetailsModel;
    } else {
      var response = jsonDecode(result.body);
      String message = response['error']['message'];
      return message;
    }
  }

  static Future<dynamic> GenerateDebitCardTransactionID(
      {required String token, required String mobileNumber, required double txnAmount, required List<Map> productDetail, required String hash, required UserMetaPreference userMetaPreference}) async {
    final url = Uri.parse(
      ApiEndPoint.test_generateTransaction,
    );

    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};
    Map transaction_summary = {
      "MOBILE_NO": mobileNumber,
      "TXN_AMOUNT": txnAmount,
      "PRODUCT_DETAILS": productDetail,
    };
    Map body = {
      "hash": hash,
      "amount": txnAmount,
      "transactionentityId": AppConstant.transection_SDK_ID,
      "transactionmodeId": AppConstant.DebitCard_TransactionMode_ID,
      "transactionstatusId": AppConstant.TransectionInProgress,
      "sourceofTxn": AppConstant.TransectionSource,
      "transaction_summary": transaction_summary
    };
    print(url);
    var result = await http.post(url, headers: header, body: jsonEncode(body));

    if (result.statusCode == 200) {
      var response = jsonDecode(result.body);
      TransactionIdDetailsModel transactionIdDetailsModel = TransactionIdDetailsModel.fromJson(response);
      print('notification count:::${response}');
      return transactionIdDetailsModel;
    } else {
      var response = jsonDecode(result.body);
      String message = response['error']['message'];
      return message;
    }
  }

  static Future<String?> generateChecksum({required String token, required String mobile, required double amount, required List product_detail}) async {
    final url = Uri.parse(
      ApiEndPoint.test_TransectionURL,
    );
    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};
    Map<String, dynamic> body = {"MOBILE_NO": mobile, "TXN_AMOUNT": amount, "PRODUCT_DETAILS": product_detail};
    String? hashKey;
    var result = await http.post(
      url,
      headers: header,
      body: json.encode(body),
    );

    if (result.statusCode == 200) {
      var response = jsonDecode(result.body);
      hashKey = response['hash'];
      return hashKey;
    }
    return hashKey;
  }

  static Future<String?> creditCardWebViewRequest({required String encodedString, required String creditCardUrl}) async {
    final url = Uri.parse(creditCardUrl);
    Map<String, dynamic> body = {"encryptData": encodedString};
    //List<int> body = utf8.encode("encryptData=$encodedString");
    var result = await http.post(url, body: body);
    if (result.statusCode == 200) {
      var response = result.body;
      print(response);
      return response;
    }
    return null;
  }

  static Future<String?> applePayWebviewRequest({required String encodedString}) async {
    final url = Uri.parse(ApiEndPoint.test_applepay_URL);
    Map<String, dynamic> body = {"data": encodedString};
    //List<int> body = utf8.encode("encryptData=$encodedString");
    var result = await http.post(url, body: body);
    if (result.statusCode == 200) {
      var response = result.body;
      print(response);
      return response;
    }
    return "";
  }

  static Future<String?> debitCardWebViewRequest({required String encodedString}) async {
    final url = Uri.parse(ApiEndPoint.test_debitCardURL);
    String body = encodedString;
    //List<int> body = utf8.encode("encryptData=$encodedString");
    var result = await http.post(url, body: body);
    if (result.statusCode == 200) {
      var response = result.body;
      print(response);
      return response;
    }
    return "";
  }

  static Future<Map?> updateTransactionCreditCard({required String token, required Map transactionDetail}) async {
    int transactionStatus = 403;
    String transactionID = "";
    int transactionAmount = 0;
    String creditCard = "";
    if (transactionDetail.containsKey('vpc_Amount')) {
      transactionAmount = int.parse(transactionDetail['vpc_Amount'] ?? "0") ~/ 10000;
    }
    if (transactionDetail.containsKey('vpc_Message')) {
      creditCard = transactionDetail['vpc_Message'];
      if (creditCard.toLowerCase() == "approved" || creditCard.toLowerCase() == "accept") {
        transactionStatus = 3;
      } else {
        transactionStatus = 2;
      }
    } else {
      transactionStatus = 2;
    }

    if (transactionDetail.containsKey('vpc_MerchTxnRef')) {
      transactionID = transactionDetail['vpc_MerchTxnRef'];
    } else {
      transactionStatus = 2;
    }

    final url = Uri.parse(
      ApiEndPoint.test_PatchTransactionURL,
    );
    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};
    Map<String, dynamic> body = {"amount": transactionAmount, "transactionno": transactionID, "transactionstatusId": transactionStatus.toString()};

    var result = await http.post(
      url,
      headers: header,
      body: json.encode(body),
    );

    if (result.statusCode == 200) {
      var response = jsonDecode(result.body);
      print(response);
      return response['data'];
    }
    var response = jsonDecode(result.body);
    print(response);
    return null;
  }
  static Future<Map?> updateTransactionDebitCard({required String token, required Map transactionDetail}) async {
    int transactionStatus = 403;
    String transactionID = "";
    int transactionAmount = 0;
    String creditCard = "";
    if (transactionDetail.containsKey('vpc_Amount')) {
      transactionAmount = int.parse(transactionDetail['vpc_Amount'] ?? "0") ~/ 10000;
    }
    if (transactionDetail.containsKey('vpc_Message')) {
      creditCard = transactionDetail['vpc_Message'];
      if (creditCard.toLowerCase() == "approved" || creditCard.toLowerCase() == "accept") {
        transactionStatus = 3;
      } else {
        transactionStatus = 2;
      }
    } else {
      transactionStatus = 2;
    }

    if (transactionDetail.containsKey('vpc_MerchTxnRef')) {
      transactionID = transactionDetail['vpc_MerchTxnRef'];
    } else {
      transactionStatus = 2;
    }

    final url = Uri.parse(
      ApiEndPoint.test_PatchTransactionURL,
    );
    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};
    Map<String, dynamic> body = {"amount": transactionAmount, "transactionno": transactionID, "transactionstatusId": transactionStatus.toString()};

    var result = await http.post(
      url,
      headers: header,
      body: json.encode(body),
    );

    if (result.statusCode == 200) {
      var response = jsonDecode(result.body);
      print(response);
      return response['data'];
    }
    var response = jsonDecode(result.body);
    print(response);
    return null;
  }
  static Future<SadadPayMinAmountCheckModel?> sadadPayMinAmountCheck({
    required String token,
  }) async {
    final url = Uri.parse(ApiEndPoint.test_minAmount);
    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};
    var result = await http.get(url, headers: header);
    if (result.statusCode == 200) {
      List<SadadPayMinAmountCheckModel> temp = sadadPayMinAmountCheckModelFromJson(result.body);
      return temp.first;
    }
    return null;
  }

  static Future<String?> sadadPayWebViewRequest({required double transactionAmount, required String token, required List products}) async {
    final url = Uri.parse(ApiEndPoint.test_SDKLoginURL);
    String strProductDetail = utf8.encode(jsonEncode(products)).toString();
    strProductDetail = strProductDetail.replaceAll("(", "[").replaceAll(")", "]");
    // Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};
    //Map<String, dynamic> body = {"TXN_AMOUNT": transactionAmount, "PRODUCT_DETAILS": strProductDetail, "ACCESS_TOKEN": ""};
    String bodyString = "TXN_AMOUNT=10.0&PRODUCT_DETAILS=$strProductDetail&ACCESS_TOKEN=$token";
    var result = await http.post(url, body: bodyString);
    if (result.statusCode == 200) {
      var response = result.body;
      print(response);
      return response;
    }
    return "";
  }

  static Future<SendOtpModel?> sadadPayLogin({required String token, required String cellnumber, required String password}) async {
    final url = Uri.parse(
      ApiEndPoint.test_sadadPayLogin,
    );
    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};
    Map<String, dynamic> body = {"cellnumber": cellnumber, "password": password};

    var result = await http.post(
      url,
      headers: header,
      body: json.encode(body),
    );

    if (result.statusCode == 200) {
      var response = jsonDecode(result.body);
      print("test_sadadPayLogin:::${response}");
      SendOtpModel sendOtpModel = sendOtpModelFromJson(jsonEncode(response));
      return sendOtpModel;
    }
    return null;
  }

  static Future<bool?> sadadPayVerifyOtp({required String token, required String otp, required String userId}) async {
    final url = Uri.parse(
      ApiEndPoint.test_verifyOtp,
    );
    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};
    Map<String, dynamic> body = {"otp": int.parse(otp), "userId": int.parse(userId)};

    var result = await http.post(
      url,
      headers: header,
      body: json.encode(body),
    );

    if (result.statusCode == 200) {
      return true;
      var response = jsonDecode(result.body);
    }
    var response = jsonDecode(result.body);
    print(response);
    return false;
  }

  static Future<bool> sadadPayResendOtp({required String token}) async {
    final url = Uri.parse(
      ApiEndPoint.test_resendOtp,
    );
    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token};

    var result = await http.get(url, headers: header);

    if (result.statusCode == 200) {
      var response = jsonDecode(result.body);
      return response['result'];
    }
    var response = jsonDecode(result.body);
    print(response);
    return false;
  }

  static Future<Map?> sadadPayTransactions(
      {required String ipAddress, required List productDetails, required String token, required String sadadId, required int userId, required double amount}) async {
    final url = Uri.parse(
      ApiEndPoint.test_transaction,
    );
    Map<String, String> header = {'Content-Type': 'application/json', "Authorization": token, "sadadId": sadadId};
    Map<String, dynamic> body = {
      "createdby": userId,
      "modifiedby": userId,
      "amount": amount,
      "senderId": userId,
      "transactionstatusId": 3,
      "transactionmodeId": 3,
      "transactionentityId": 9,
      "transaction_summary": productDetails,
      "txnip": ipAddress
    };

    var result = await http.post(
      url,
      headers: header,
      body: json.encode(body),
    );

    if (result.statusCode == 200) {
      var response = jsonDecode(result.body);
      return response;
    }
    return null;
  }
}
